__author__ = "xTrinch"
__email__ = "mojca.rojko@gmail.com"
__version__ = "1.0.7"


class NotificationError(Exception):
    pass


try:
    from django import VERSION as DJANGO_VERSION

    if DJANGO_VERSION < (3, 2):
        default_app_config = "fcm_django.apps.FcmDjangoConfig"
except ImportError:
    pass



# Generated by Django 1.9.13 on 2017-08-08 11:45

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

FCM_DJANGO_SETTINGS = getattr(settings, "FCM_DJANGO_SETTINGS", {})
USER_MODEL = FCM_DJANGO_SETTINGS.get("USER_MODEL", settings.AUTH_USER_MODEL)


class Migration(migrations.Migration):

    dependencies = [
        ("fcm_django", "0004_auto_20181128_1642"),
    ]

    operations = [
        migrations.AlterField(
            model_name="fcmdevice",
            name="user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to=USER_MODEL,
            ),
        ),
    ]



# Generated by Django 3.1.12 on 2021-12-24 12:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fcm_django', '0007_auto_20211001_1440'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fcmdevice',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



# Generated by Django 1.9.8 on 2016-08-05 15:40

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="FCMDevice",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True, max_length=255, null=True, verbose_name="Name"
                    ),
                ),
                (
                    "active",
                    models.BooleanField(
                        default=True,
                        help_text="Inactive devices will not be sent notifications",
                        verbose_name="Is active",
                    ),
                ),
                (
                    "date_created",
                    models.DateTimeField(
                        auto_now_add=True, null=True, verbose_name="Creation date"
                    ),
                ),
                (
                    "device_id",
                    models.UUIDField(
                        blank=True,
                        db_index=True,
                        help_text="GUID()",
                        null=True,
                        verbose_name="Device ID",
                    ),
                ),
                (
                    "registration_id",
                    models.TextField(verbose_name="Registration token"),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[("ios", "ios"), ("android", "android")], max_length=10
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "FCM device",
            },
        ),
    ]



from tastypie.authentication import BasicAuthentication
from tastypie.authorization import Authorization
from tastypie.resources import ModelResource

from fcm_django.models import FCMDevice


class FCMDeviceResource(ModelResource):
    class Meta:
        authorization = Authorization()
        queryset = FCMDevice.objects.all()
        resource_name = "device/apns"


class APNSDeviceAuthenticatedResource(FCMDeviceResource):
    # user = ForeignKey(UserResource, "user")

    class Meta(FCMDeviceResource.Meta):
        authentication = BasicAuthentication()

    # authorization = SameUserAuthorization()

    def obj_create(self, bundle, **kwargs):
        # See https://github.com/toastdriven/django-tastypie/issues/854
        return super().obj_create(bundle, user=bundle.request.user, **kwargs)



from django.apps import AppConfig

from fcm_django.settings import FCM_DJANGO_SETTINGS as SETTINGS


class FcmDjangoConfig(AppConfig):
    name = "fcm_django"
    verbose_name = SETTINGS["APP_VERBOSE_NAME"]
    default_auto_field = "django.db.models.BigAutoField"



from django.db.models import Q
from rest_framework import permissions, status
from rest_framework.mixins import CreateModelMixin
from rest_framework.response import Response
from rest_framework.serializers import ModelSerializer, Serializer, ValidationError
from rest_framework.viewsets import GenericViewSet, ModelViewSet

from fcm_django.models import FCMDevice
from fcm_django.settings import FCM_DJANGO_SETTINGS as SETTINGS


# Serializers
class DeviceSerializerMixin(ModelSerializer):
    class Meta:
        fields = (
            "id",
            "name",
            "registration_id",
            "device_id",
            "active",
            "date_created",
            "type",
        )
        read_only_fields = ("date_created",)

        extra_kwargs = {"active": {"default": True}}


class UniqueRegistrationSerializerMixin(Serializer):
    def validate(self, attrs):
        devices = None
        primary_key = None
        request_method = None

        if self.initial_data.get("registration_id", None):
            if self.instance:
                request_method = "update"
                primary_key = self.instance.id
            else:
                request_method = "create"
        else:
            if self.context["request"].method in ["PUT", "PATCH"]:
                request_method = "update"
                primary_key = self.instance.id
            elif self.context["request"].method == "POST":
                request_method = "create"

        Device = self.Meta.model
        # if request authenticated, unique together with registration_id and
        # user
        user = self.context["request"].user
        registration_id = attrs.get("registration_id")

        if request_method == "update":
            if registration_id:
                if user is not None and user.is_authenticated:
                    devices = Device.objects.filter(
                        registration_id=registration_id
                    ).exclude(id=primary_key)
                    if attrs.get("active", False):
                        devices.filter(~Q(user=user)).update(active=False)
                    devices = devices.filter(user=user)
                else:
                    devices = Device.objects.filter(
                        registration_id=registration_id
                    ).exclude(id=primary_key)
        elif request_method == "create":
            if user is not None and user.is_authenticated:
                devices = Device.objects.filter(registration_id=registration_id)
                devices.filter(~Q(user=user)).update(active=False)
                devices = devices.filter(user=user, active=True)
            else:
                devices = Device.objects.filter(registration_id=registration_id)

        if devices:
            raise ValidationError({"registration_id": "This field must be unique."})
        return attrs


class FCMDeviceSerializer(ModelSerializer, UniqueRegistrationSerializerMixin):
    class Meta(DeviceSerializerMixin.Meta):
        model = FCMDevice

        extra_kwargs = {"id": {"read_only": True, "required": False}}
        extra_kwargs.update(DeviceSerializerMixin.Meta.extra_kwargs)


# Permissions
class IsOwner(permissions.BasePermission):
    def has_object_permission(self, request, view, obj):
        # must be the owner to view the object
        return obj.user == request.user


# Mixins
class DeviceViewSetMixin:
    lookup_field = "registration_id"

    def create(self, request, *args, **kwargs):
        serializer = None
        is_update = False
        if (
            SETTINGS.get("UPDATE_ON_DUPLICATE_REG_ID")
            and "registration_id" in request.data
        ):
            instance = self.queryset.model.objects.filter(
                registration_id=request.data["registration_id"]
            ).first()
            if instance:
                serializer = self.get_serializer(instance, data=request.data)
                is_update = True
        if not serializer:
            serializer = self.get_serializer(data=request.data)

        serializer.is_valid(raise_exception=True)
        if is_update:
            self.perform_update(serializer)
            return Response(serializer.data)
        else:
            self.perform_create(serializer)
            headers = self.get_success_headers(serializer.data)
            return Response(
                serializer.data, status=status.HTTP_201_CREATED, headers=headers
            )

    def perform_create(self, serializer):
        if self.request.user.is_authenticated:
            if SETTINGS["ONE_DEVICE_PER_USER"] and self.request.data.get(
                "active", True
            ):
                FCMDevice.objects.filter(user=self.request.user).update(active=False)
            return serializer.save(user=self.request.user)
        return serializer.save()

    def perform_update(self, serializer):
        if self.request.user.is_authenticated:
            if SETTINGS["ONE_DEVICE_PER_USER"] and self.request.data.get(
                "active", False
            ):
                FCMDevice.objects.filter(user=self.request.user).update(active=False)

            return serializer.save(user=self.request.user)
        return serializer.save()


class AuthorizedMixin:
    permission_classes = (permissions.IsAuthenticated, IsOwner)

    def get_queryset(self):
        # filter all devices to only those belonging to the current user
        return self.queryset.filter(user=self.request.user)


# ViewSets
class FCMDeviceViewSet(DeviceViewSetMixin, ModelViewSet):
    queryset = FCMDevice.objects.all()
    serializer_class = FCMDeviceSerializer


class FCMDeviceCreateOnlyViewSet(DeviceViewSetMixin, CreateModelMixin, GenericViewSet):
    queryset = FCMDevice.objects.all()
    serializer_class = FCMDeviceSerializer


class FCMDeviceAuthorizedViewSet(AuthorizedMixin, FCMDeviceViewSet):
    pass
